
#include <errno.h>

#include <gkrellm2/gkrellm.h>

#define STYLE_NAME "gkcpu"

#define SYSFS_TREE     "/sys/devices/system/cpu/cpu0/cpufreq/"
#define SYSFS_MAXSPEED "scaling_max_freq"
#define SYSFS_GETSPEED "scaling_cur_freq"

// begin changes jb

// on my system ist THRM, not THM0
// #define ACPI_CPU_TEMP_FILE "/proc/acpi/thermal_zone/THM0/temperature"
#define ACPI_CPU_TEMP_FILE "/proc/acpi/thermal_zone/THRM/temperature"

// end changes jb


static GkrellmMonitor *mon_cpufreq;
static GkrellmPanel *pcpufreq;
static GkrellmKrell *krell;
static GkrellmDecal *text;
static GkrellmTicks   *pGK;

static gint cpufreq_style_id;

static char buffer[1024];

static gint read_file( char *fname) {
   int fd, err, numread;
   
   if ((fd = open( fname, O_RDONLY)) == -1) {
      err = errno;
      perror( "couldn't open file");
      return err;
   }
   
   if ((numread = read( fd, buffer, sizeof( buffer)-1)) < 0) {
      err = errno;
      perror( "couldn't read file");
      return err;
   }
   
   buffer[numread] = '\0';
   close( fd);
   
   return 0;
}

static gint get_max_speed() {
   gint err;
   char filename[256];
   gint maxfreq = 0;
   
   char *buf;
   gint *freq;

   strcpy( filename, SYSFS_TREE);
   strcat( filename, SYSFS_MAXSPEED);
   
   if ((err = read_file( filename)) < 0) {
      perror( "error reading max frequency");
      return err;
   }
   
   sscanf( buffer, "%d", &maxfreq);
   
   return maxfreq;
}

static gint get_cpu_speed() {
   gint err;
   char filename[256];
   gint freq;
   
   strcpy( filename, SYSFS_TREE);
   strcat( filename, SYSFS_GETSPEED);
  
   if ((err = read_file( filename)) < 0) {
      perror( "error reading available frequencies");
      return err;
   }
   
   sscanf( buffer, "%d", &freq);
   
   return freq;
}

static gint get_cpu_temp() {
   gint err;
   gint temp;
   
   
   if ((err = read_file( ACPI_CPU_TEMP_FILE)) < 0) {
      perror( "error reading available frequencies");
      return err;
   }
   
   sscanf( buffer, "%*s %d", &temp);
   
   return temp;
}

static void create_cpufreq( GtkWidget *vbox, gint first_create) {

   GkrellmStyle     *style;
   GkrellmTextstyle *textstyle;
   GkrellmPiximage  *krell_image;
   
   if (first_create) {
      pcpufreq = gkrellm_panel_new0();
   }
   style = gkrellm_meter_style( cpufreq_style_id);
   textstyle = gkrellm_meter_textstyle( cpufreq_style_id);
   
   krell_image = gkrellm_krell_meter_piximage( cpufreq_style_id);
   krell = gkrellm_create_krell( pcpufreq, krell_image, style);
   
   text = gkrellm_create_decal_text( pcpufreq, "16.0 GHz / 12", textstyle, style, 3, 10, 0);
   
   gkrellm_set_krell_margins( pcpufreq, krell, 3, 2);
   gkrellm_monotonic_krell_values( krell, FALSE);
   gkrellm_set_krell_full_scale( krell, get_max_speed(), 1);
   
   gkrellm_panel_configure( pcpufreq, NULL, style);
   gkrellm_panel_create( vbox, mon_cpufreq, pcpufreq);
}

static void update_cpufreq( void) {
   // begin chages by jb
   // gchar buf[32];
   // end changes by jb
   gchar buf[48];
   gchar *buf2 = buf;
   gint speed;
   gfloat fspeed;
   gint temp;
   
   if(!pGK->second_tick) {
   	   return;
   }
   
   speed = get_cpu_speed();
   
   // begin chages by jb
   
   // I don't need Thermal Zone Info here!
   //temp = get_cpu_temp();
   
   // I want it done differently!
   // fspeed = (gfloat) speed / 1000000;
   // sprintf( buf, "%.1f / %d°", fspeed, temp);
   // if (fspeed >= 10.0) buf2++;
   
   // I'd really like Mhz, thank you, and no Thermal Info!
   fspeed = (gfloat) speed / 1000;
   sprintf( buf, "%5.1f Mhz", fspeed);
   
   // end changes by jb
   
   gkrellm_draw_decal_text( pcpufreq, text, buf2, -1);
//   gkrellm_update_krell( pcpufreq, krell, 0);
   gkrellm_update_krell( pcpufreq, krell, get_cpu_speed());

   gkrellm_draw_panel_layers( pcpufreq);
}

/*
 * CONFIGURATION PART
 */
 
#define PLUGIN_CONFIG_KEYWORD STYLE_NAME

static gchar   *plugin_info_text[] = {
   "<h>gkcpu monitor\n",
   "\tMore text will occur here in a later version..."
};
 
static void create_config_tabs(GtkWidget *tab_vbox)
   {
   GtkWidget      *tabs;
   GtkWidget      *vbox;
   GtkWidget      *text;
   gint        i;

// create tabs

   tabs = gtk_notebook_new();
   gtk_notebook_set_tab_pos( GTK_NOTEBOOK(tabs), GTK_POS_TOP);
   gtk_box_pack_start( GTK_BOX( tab_vbox), tabs, TRUE, TRUE, 0);

// config tab

   vbox = gkrellm_gtk_framed_notebook_page(tabs, "Setup");

// info tab

   vbox = gkrellm_gtk_framed_notebook_page( tabs, "Info");
   text = gkrellm_gtk_scrolled_text_view( vbox, NULL,
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   for (i = 0; i < sizeof(plugin_info_text)/sizeof(gchar *); ++i)
      gkrellm_gtk_text_view_append( text, plugin_info_text[i]);
}

static void load_config( gchar *arg) {
	// TODO
}

static void save_config( FILE *f) {
	// TODO
}

static void apply_config() {
	// TODO
}

GkrellmMonitor monitor_cpufreq = {
   STYLE_NAME,          /* Name, for config tab.        */
   0,                  /* Id,  0 if a plugin           */
   create_cpufreq,     /* The create_plugin() function */
   update_cpufreq,     /* The update_plugin() function */
   create_config_tabs, /* The create_plugin_tab() config function */
   apply_config,       /* The apply_plugin_config() function      */

   save_config,        /* The save_plugin_config() function  */
   load_config,        /* The load_plugin_config() function  */
   PLUGIN_CONFIG_KEYWORD, /* config keyword                     */

   NULL,               /* Undefined 2  */
   NULL,               /* Undefined 1  */
   NULL,               /* Undefined 0  */

   MON_MAIL,           /* Insert plugin before this monitor.       */
   NULL,               /* Handle if a plugin, filled in by GKrellM */
   NULL                /* path if a plugin, filled in by GKrellM   */
};
    
GkrellmMonitor *
gkrellm_init_plugin(void) {

   pGK = gkrellm_ticks();
   cpufreq_style_id = gkrellm_add_meter_style( &monitor_cpufreq,
                                                STYLE_NAME);
   
   mon_cpufreq = &monitor_cpufreq;
   return mon_cpufreq;
}
